Package: HDF5Array
Title: HDF5 backend for DelayedArray objects
Description: Implement the HDF5Array, H5SparseMatrix, H5ADMatrix, and
	TENxMatrix classes, 4 convenient and memory-efficient array-like
	containers for representing and manipulating either: (1) a conventional
	(a.k.a. dense) HDF5 dataset, (2) an HDF5 sparse matrix (stored in
	CSR/CSC/Yale format), (3) the central matrix of an h5ad file (or any
	matrix in the /layers group), or (4) a 10x Genomics sparse matrix.
	All these containers are DelayedArray extensions and thus support all
	operations (delayed or block-processed) supported by DelayedArray
	objects.
biocViews: Infrastructure, DataRepresentation, DataImport, Sequencing,
        RNASeq, Coverage, Annotation, GenomeAnnotation, SingleCell,
        ImmunoOncology
URL: https://bioconductor.org/packages/HDF5Array
BugReports: https://github.com/Bioconductor/HDF5Array/issues
Version: 1.30.1
License: Artistic-2.0
Encoding: UTF-8
Author: Hervé Pagès
Maintainer: Hervé Pagès <hpages.on.github@gmail.com>
Depends: R (>= 3.4), methods, DelayedArray (>= 0.27.2), rhdf5 (>=
        2.31.6)
Imports: utils, stats, tools, Matrix, rhdf5filters, BiocGenerics (>=
        0.31.5), S4Vectors, IRanges, S4Arrays (>= 1.1.1)
LinkingTo: S4Vectors (>= 0.27.13), Rhdf5lib
SystemRequirements: GNU make
Suggests: BiocParallel, GenomicRanges, SummarizedExperiment (>=
        1.15.1), h5vcData, ExperimentHub, TENxBrainData, zellkonverter,
        GenomicFeatures, RUnit, SingleCellExperiment,
        DelayedMatrixStats, genefilter
Collate: utils.R H5File-class.R h5ls.R H5DSetDescriptor-class.R
        h5utils.R h5dimscales.R uaselection.R h5mread.R
        h5mread_from_reshaped.R h5writeDimnames.R h5summarize.R
        HDF5ArraySeed-class.R HDF5Array-class.R
        ReshapedHDF5ArraySeed-class.R ReshapedHDF5Array-class.R
        dump-management.R writeHDF5Array.R
        saveHDF5SummarizedExperiment.R H5SparseMatrixSeed-class.R
        H5SparseMatrix-class.R H5ADMatrixSeed-class.R
        H5ADMatrix-class.R TENxMatrixSeed-class.R TENxMatrix-class.R
        writeTENxMatrix.R zzz.R
git_url: https://git.bioconductor.org/packages/HDF5Array
git_branch: RELEASE_3_18
git_last_commit: 1aee703
git_last_commit_date: 2024-02-14
Repository: Bioconductor 3.18
Date/Publication: 2024-02-14
NeedsCompilation: yes
Packaged: 2024-02-14 22:24:27 UTC; biocbuild
